import React from 'react';
import { Checkbox } from 'antd';
import { emitter, request } from '../utils';
import config from '../../config';
import styles from './index.less';

export default class AtMessageContainer extends React.Component {

  constructor(props) {
    super(props);

    this.state = {
      users: [],
      selectIndex: 0,
      noticeStatus: true
    }
  }

  componentDidMount() {
    this.fetchUsersList();
    emitter.addListener('textChange', this.fetchUsersList);
    window.addEventListener("keydown", this.keydown);
  }

  componentWillUnmount() {
    emitter.removeListener('textChange', this.fetchUsersList);
    window.removeEventListener("keydown", this.keydown);
  }

  fetchUsersList = (text) => {
    request(`${config.DOMAIN}/scene/api/fetchUsers?name=${text || ''}`)
      .then(data => {
        if (data) {
          this.setState({
            users: data || []
          });
        }
      })
  }

  onHandleSelect = (item) => {
    const { noticeStatus } = this.state;
    const { onSelect } = this.props;
    onSelect && onSelect([item], noticeStatus);
  }

  onChangeNoticeStatus = (e) => {
    const checkedValue = e.currentTarget.checked;
    this.setState({
      noticeStatus: checkedValue
    });
  }

  keydown = (e) => {
    const { selectIndex, users } = this.state;
    // 下
    if (40 === e.keyCode) {
      if (selectIndex < users.length - 1) {
        this.setState({
          selectIndex: selectIndex + 1
        })
      }
    }

    // 上
    if (38 === e.keyCode) {
      if (selectIndex > 0) {
        this.setState({
          selectIndex: selectIndex - 1
        })
      }
    }

    // 回车
    if (13 === e.keyCode) {
      const { noticeStatus } = this.state;
      const { onSelect } = this.props;
      onSelect && onSelect([users[selectIndex]], noticeStatus);
    }

    // ESC
    if (27 === e.keyCode) {
      const { noticeStatus } = this.state;
      const { onSelect } = this.props;
      onSelect && onSelect();
    }
  }

  render() {
    const { users, noticeStatus, selectIndex } = this.state;

    return (
      <div className={styles.container}>
        <div className={styles.titleContainer}>
          <div className={styles.title}>你可能想提及</div>
          <Checkbox className={styles.noticeCheckbox} onClick={this.onChangeNoticeStatus} checked={noticeStatus}>
            <span className={styles.noticeCheckboxTitle}>消息提醒对方</span>
          </Checkbox>
        </div>
        {
          users.map((item, index) => {
            return (
              <div
                key={item.name}
                className={`${styles.userContainer} ${selectIndex === index ? styles.selected : ''}`}
                onClick={this.onHandleSelect.bind(this, item)}
              >
                <div className={styles.headSculpture}>{item.name.slice(item.name.length - 2)}</div>
                <span className={styles.name}>{item.name}</span>
              </div>
            )
          })
        }
        {
          users.length === 0 && <div className={styles.noUserMessage}>暂无</div>
        }
      </div>
    )
  }
}