import React from 'react';
import ReactDom from 'react-dom';
import { emitter, request } from './utils';
import config from '../config';
import AtContainer from './AtContainer/index';
import UserContainer from './UserContainer/index';

// 避免antd.less影响全局样式
import 'antd/es/message/style/index.css';
import 'antd/es/checkbox/style/index.css';

const EVENT_PREFIX = 'PLUGIN_'
const EVENTS = {
  RENDER: `${EVENT_PREFIX}RENDER`,
  SET_CONTAINER_POSITION: `${EVENT_PREFIX}SET_CONTAINER_POSITION`,
  INPUT: `${EVENT_PREFIX}INPUT`,
  CANCEL: `${EVENT_PREFIX}CANCEL`,
  CLICK: `${EVENT_PREFIX}CLICK`,
  CONFIRM: `${EVENT_PREFIX}CONFIRM`,
  ADD: `${EVENT_PREFIX}ADD`
}
const pluginType = 'AT'
const keyBoardEvent = {
  enter: 13,
  up: 38,
  down: 40,
  left: 37,
  right: 39,
  esc: 27
}

class Message {
  constructor(WPS_PLUGIN_API) {
    this.$api = {
      ...WPS_PLUGIN_API.layout, // 获取需要用到的正文容器相关api
      getContainer: () => {
        const container = WPS_PLUGIN_API.layout.getPluginContainer()
        return container
      },
      message: (event, detail = {}) => {
        // 给正文发送消息的方法
        WPS_PLUGIN_API.messagePush({
          pluginType,
          pluginEvent: event,
          detail
        })
      }
    }
    this.user = null // 当前用户,{userId: 123, name: 'zzz'}
    this.curCardUUID = '' // 当前展示卡片的uuid
    this.onHide = this.onHide.bind(this)
    window.parent.postMessage(JSON.stringify({eventName: 'at_init'}), "*");
    window.addEventListener('message', this.userListener)
  }
  userListener = (e) => {
    let data = {};
    try {
      data = JSON.parse(e.data);
    } catch (e) {
    }

    if (data.type === 'at_user_info') {
      this.user = data.user;
      window.removeEventListener('message', this.userListener)
    }
  }
  messageGet(params) {
    console.error('get message', params)
    if (!this.user) return // 未登录不响应
  
    const { pluginEvent, detail } = params
  
    const EventsMap = {
      [EVENTS.RENDER]: () => this.onRender(detail),
      [EVENTS.SET_CONTAINER_POSITION]: () => this.changePos(detail),
      [EVENTS.INPUT]: () => this.onInput(detail),
      [EVENTS.CANCEL]: () => this.onHide(),
      [EVENTS.CLICK]: () => this.onClick(detail),
      [EVENTS.ADD]: () => this.onAdd(detail)
    }
    EventsMap[pluginEvent] && EventsMap[pluginEvent]()
  }
    // 正文通知展示@面板
    onRender(detail) {
      console.log("AT插件onRender执行", detail)
      const container = this.$api.getContainer();
      const { left, top } = this.setContainerPlace(detail);
      this.$api.changerContanierTopAndLeft({top: `${top}px`, left: `${left}px`})
      ReactDom.render(<AtContainer onSelect={this.onSelect.bind(this)}/>, container);

      this.$api.message('PLUGIN_ACTION_CONFIG', {
        keyboard: [
          keyBoardEvent.enter,
          keyBoardEvent.up,
          keyBoardEvent.down,
          keyBoardEvent.left,
          keyBoardEvent.right,
          keyBoardEvent.esc
        ].map(e => ({keyCode: e}))
      });
      // this.$api.message('PLUGIN_ACTION_CONFIG')
    }
  
    // 改变容器位置
    changePos(detail) {
      console.log("AT插件changePos执行");
      const { left, top } = this.setContainerPlace(detail);
      this.$api.changerContanierTopAndLeft({top: `${top}px`, left: `${left}px`})
    }

    // 设置容器位置
    setContainerPlace(detail) {
      const { begin, end = begin, bound } = detail;
      const container = this.$api.getContainer();
      let left = end.x + container.clientWidth > bound.right ? 
                  begin.x - container.clientWidth : end.x;
      let top = begin.y + container.clientHeight > bound.bottom ? 
                  begin.y - container.clientHeight : begin.y + begin.offsetY;
      return {left, top}
    }

  
    // 正文输入回调
    onInput({ text, isComposing }) {
      console.log("AT插件onInput执行")
      if (isComposing) return // 预输入的时候不处理
      text = text.replace(/@/g, '')
      if (/\s$/.test(text) || text.length > 15) return this.onHide()
      // TODO 筛选用户列表
      emitter.emit('textChange', text);
    }
  
    // 插件隐藏、取消事件
    onHide() {
      console.log("AT插件onHide执行")
      // TODO 关闭@面板
      const container = this.$api.getContainer();
      ReactDom.unmountComponentAtNode(container);
      this.$api.clearContainer() // 清空容器
      this.$api.message(EVENTS.CANCEL) // 通知正文
    }
  
    // 正文点击事件
    onClick(detail = {}) {
      console.log("AT插件onClick执行")
      // TODO 移动端点击名字不响应
      const isMobileMode = false;
      if (isMobileMode) return
  
      const { data = {}, isOutside } = detail
      const { uuid } = data
      // TODO 含有uuid则表明此时是点击@的内容，根据uuid展示对应卡片
      if (isOutside || uuid === this.curCardUUID) {
        this.curCardUUID = '';
        return this.onHide()
      } else {
        if (!uuid) return;
        this.curCardUUID = uuid;
        const container = this.$api.getContainer();
        ReactDom.render(<UserContainer detail={data}/>, container, () => {
          const { left, top } = this.setContainerPlace(detail);
          this.$api.changerContanierTopAndLeft({top: `${top}px`, left: `${left}px`})
        })
      }
    }

    // 是否需要发送消息
    get isSend() {
      return true
    }
  
    // 选中成员
    onSelect(users, noticeStatus) {
      console.log("AT插件onSelect执行")
      if (!users) return this.onHide()
      !Array.isArray(users) && (users = [users])
      this.$api.message(EVENTS.CONFIRM, {
        // 只发送约定好的属性
        list: users.map((user) => {
          const { uuid, userid, avatar, name } = user
          return {
            uuid,
            userid,
            name,
            avatar,
            appid: "your appid"
          }
        }),
        isSend: this.isSend
      })

      // 存储消息至服务端，消息提醒不选时不设置消息
      if (!noticeStatus) return;
      users.map(user => {
        if (user.id === this.user.userId) return;
        request(`${config.DOMAIN}/scene/api/saveMessage`,{
          fileInfo: window.fileInfo,
          user: user,
          creater: this.user
        }, 'POST');
      })
     
    }

    // 新增@人成员
    onAdd(detail) {
      const { data, isRecovery } = detail;
      // 消息推送至接收人
      const message = {
        eventName: 'notice_message_at_info',
      }

      // isRecovery: 是否回退@操作
      if (isRecovery) {
        request(`${config.DOMAIN}/scene/api/changeMessageStatus?uuid=${data.uuid}`).then(() => {
          window.parent.postMessage(JSON.stringify(message), '*')
        });
      } else {
        window.parent.postMessage(JSON.stringify(message), '*')
      }
    }
}

class At {
  // 插件提供给事件流，事件流用此API推送消息给插件
  constructor(WPS_PLUGIN_API) {
    console.error('load AT success', WPS_PLUGIN_API)
    const message = new Message(WPS_PLUGIN_API)
    WPS_PLUGIN_API.subscribe(message)
  }

  // 插件初始化方法，
  async init(params = {}) {
    console.error("AT插件init执行")
    const { fid, openPermission = 'write', } = params
    return true
  }

}

(function() {
  // 通知插件中心插件已加载
  window.__WPSPLUGIN__.loader.addPlugin({
    // ${WEB_OFFICE_PLUGIN_ID}是占用符，上传后会被替换成具体的id
    id: "APP6D7DFC94",
    pluginType: "AT",
    pluginObj: At,
  });
})();