```js
// PDF对象
class pdf {
  wsServer: WebSocket;
  meetingInfo: any;
  wspApplication: any;
  optionType = {
    ZOOM: {name: 'ZoomUpdated'},  // 缩放
    SCROLL: {name: 'Scroll'}, // 滚动
    PAGECHANGE: {name: 'CurrentPageChange'}, // 页面切换
    PAGESTARTPLAY: {name: 'StartPlay'}, // 进入播放
    PAGEENDPLAY: {name: 'EndPlay'} // 退出播放
  };
  cacheScroll: any;

  constructor(wsServer: WebSocket, meetingInfo: any, wspApplication: any) {
    this.wsServer = wsServer;
    this.meetingInfo = meetingInfo;
    this.wspApplication = wspApplication;
  }

  // 初始化主讲人PDF事件监听
  speakerInit() {
    Object.keys(this.optionType).forEach((key) => {
      this.wspApplication.Sub[this.optionType[key].name] = async (e: any) => {
        let message = e;
        if (this.optionType[key].name === this.optionType.SCROLL.name) {
          const zoom = await this.wspApplication.ActivePDF.Zoom;
          message = {scroll: e, zoom};
        } else if (this.optionType[key].name === this.optionType.ZOOM.name) {
          const scroll = await this.wspApplication.ActivePDF.Scroll;
          message = {scroll, zoom: e};
        }
        this.postMessage({type: this.optionType[key].name, value: message});
      }
    })
  }

  async sendInitInfo() {
    const zoom = await this.wspApplication.ActivePDF.Zoom;
    const scroll = await this.wspApplication.ActivePDF.Scroll;
    const playMode = await this.wspApplication.ActivePDF.PlayMode;
    this.postMessage({
      type: 'init', 
      value: {
        [this.optionType.ZOOM.name]: {zoom, scroll},
        ...(playMode?
          {[this.optionType.PAGESTARTPLAY.name]: true}:
          {[this.optionType.PAGEENDPLAY.name]: true}
        )
      }
    });
  }

  // 初始化听众响应操作
  async listenerInit(optionInfo: any) {
    switch(optionInfo.type) {
      case this.optionType.SCROLL.name: {
        const zoom = await this.wspApplication.ActivePDF.Zoom;
        const x = optionInfo.value.scroll.ScrollX / optionInfo.value.zoom * zoom;
        const y = optionInfo.value.scroll.ScrollY / optionInfo.value.zoom * zoom;
        this.wspApplication.ActivePDF.ScrollTo(x, y);
        this.cacheScroll = {x, y};
        break;
      }
      case this.optionType.ZOOM.name:
        this.wspApplication.ActivePDF.Zoom = optionInfo.value.zoom;
         // 缩放后设置滚动位置
        this.wspApplication.ActivePDF.ScrollTo(optionInfo.value.scroll.x, optionInfo.value.scroll.y);
        break;
      case this.optionType.PAGECHANGE.name: {
        // 通过滚动判断，非连页模式下切页才有效
        setTimeout(async () => {
          const scroll = await window.listenerApp.Application.ActivePDF.Scroll;
          if (Math.abs(scroll.x - this.cacheScroll.x) < 1 && Math.abs(scroll.y - this.cacheScroll.y) < 1) {
            return;
          }
          this.wspApplication.ActivePDF.JumpToPage({PageNum: optionInfo.value + 1});
        })
        break;
      }
      case this.optionType.PAGESTARTPLAY.name:
        this.wspApplication.ActivePDF.StartPlay('active',true,true)
        break;
      case this.optionType.PAGEENDPLAY.name:
        this.wspApplication.ActivePDF.EndPlay()
        break;
      default:
        break;
    }
  }

  // 消息推送
  postMessage = (data: any) => {
    const { meetId, user } = this.meetingInfo;
    this.wsServer.send(JSON.stringify({id: user.id, meetId, data}));
  }
}
```