```js
// 演示对象
class presentation {
  wsServer: WebSocket;
  wspApplication: any;
  meetingInfo: any;
  cacheInfo = {}; // 文档操作缓存
  optionType = {
    SLIDESHOWONNEXT: {name: 'SlideShowOnNext'}, // 播放模式下一步触发
    SLIDESHOWONPREVIOUS: {name: 'SlideShowOnPrevious'}, // 播放模式上一步触发
    SLIDEMEDIACHANGED: {name: 'SlideMediaChanged'}, // 视频播放状态改变时触发
    SLIDEPLAYERCHANGE: {name: 'SlidePlayerChange'}, // 播放状态改变时触发
    SLIDELASERPENINKPOINTSCHANGED: {name: 'SlideLaserPenInkPointsChanged'}, // 发送激光笔的墨迹
    SLIDESHOWBEGIN: {name: 'SlideShowBegin'}, // 进入播放 
    SLIDESHOWEND: {name: 'SlideShowEnd'}, // 退出播放 
    SLIDEINKVISIBLE: {name: 'SlideInkVisible'}, // 是否显示标注
    SLIDEINKTOOLBARVISIBLE: {name: 'SlideInkToolbarVisible'}, // 是否使用激光笔和标注
  };

  constructor(wsServer: WebSocket, meetingInfo: any, wspApplication: any) {
    this.wsServer = wsServer;
    this.meetingInfo = meetingInfo;
    this.wspApplication = wspApplication;
  }

  // 初始化主讲人演示事件监听
  speakerInit() {
    Object.keys(this.optionType).forEach(async (key) => {
      this.wspApplication.Sub[this.optionType[key].name] = async (e: any) => {
        let message = e;
        if (this.optionType[key].name === this.optionType.SLIDESHOWBEGIN.name) {
          // 初始化播放页码与动画
          const slideIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideIndex
          message = {slideIndex, clickIndex: 0};
        }
        this.postMessage({type: this.optionType[key].name, value: message})
        this.cacheInfo[this.optionType[key].name] = message;
        // 播放模式隐藏右键菜单\hover\链接
        if (this.optionType[key].name === this.optionType.SLIDESHOWBEGIN.name) {
          delete this.cacheInfo[this.optionType.SLIDESHOWEND.name];
          await this._setMenusVisible(false);
        } else if (this.optionType[key].name === this.optionType.SLIDESHOWEND.name) {
          delete this.cacheInfo[this.optionType.SLIDESHOWBEGIN.name];
          await this._setMenusVisible(true);
        }
      }
    })
  }
  /**
   * 主讲人初始化消息推送至听众
   * @param isPlay 是否开启播放模式
   */
  async sendInitInfo(isPlay?: boolean) {
    // 演示文档初始化后自动进入播放模式
    const params = {};
    if (isPlay) {
      await this.wspApplication.ActivePresentation.SlideShowSettings.Run();
      const slideIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideIndex;
      const clickIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.GetClickIndex();
      this.cacheInfo[this.optionType.SLIDESHOWBEGIN.name] = {slideIndex, clickIndex};
      params[this.optionType.SLIDESHOWBEGIN.name] = {slideIndex, clickIndex};
    } else if (this.cacheInfo[this.optionType.SLIDESHOWBEGIN.name]) {
      const slideIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideIndex;
      const clickIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.GetClickIndex();
      this.cacheInfo[this.optionType.SLIDESHOWBEGIN.name] = {slideIndex, clickIndex};
      params[this.optionType.SLIDESHOWBEGIN.name] = {slideIndex, clickIndex};
    }
    
    // 除页面页码、动画设置外，其它初始化根据缓存设置
    const filterKey = [
      this.optionType.SLIDESHOWONNEXT.name,
      this.optionType.SLIDESHOWONPREVIOUS.name,
      this.optionType.SLIDEPLAYERCHANGE.name
    ]; 
    Object.keys(this.cacheInfo).forEach(key => {
      if (filterKey.indexOf(key) < 0 ) {
        params[key] = this.cacheInfo[key];
      }
    })
    this.postMessage({type: 'init', value: params});
  }

  // 初始化听众响应操作
  async listenerInit(optionInfo: any) {
    switch(optionInfo.type) {
      case this.optionType.SLIDESHOWONNEXT.name: {
        //await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoNextClick()
        break;
      }
      case this.optionType.SLIDESHOWONPREVIOUS.name: {
        //await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoPreClick()
        break;
      }
      case this.optionType.SLIDEPLAYERCHANGE.name: {
        const playMode = await this.wspApplication.ActivePresentation.SlideShowWindow.View.State;
        const slideId = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideID;
        const slideIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideIndex;
        if (slideId === optionInfo.value.Data.slideId
              && slideIndex !== optionInfo.value.Data.slideIndex + 1 || playMode !== 'play') {
          return;
        }
        
        if (optionInfo.value.Data.action === 'forward') {
           await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoNextClick()
          //  this._setPageAndAnimate(optionInfo);
        }
        if (optionInfo.value.Data.action === 'backward') {
          await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoPreClick()
          // this._setPageAndAnimate(optionInfo);
        }
        if (optionInfo.value.Data.action === 'switchTo'
            || optionInfo.value.Data.action === 'effectFinish'
            && optionInfo.value.Data.isLastEffect) {
          this._setPageAndAnimate(optionInfo);
        }
        break;
      }
      case this.optionType.SLIDESHOWBEGIN.name:
        await this.wspApplication.ActivePresentation.SlideShowSettings.Run()
        await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoSlide(optionInfo.value.slideIndex)
        await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoClick(optionInfo.value.clickIndex)
        await this._setMenusVisible(false);
        break;
      case this.optionType.SLIDESHOWEND.name:
        await this.wspApplication.ActivePresentation.SlideShowWindow.View.Exit()
        await this._setMenusVisible(true);
        break;
      case this.optionType.SLIDEINKVISIBLE.name:
        this.wspApplication.ActivePresentation.SlideShowWindow.View.PointerVisible = optionInfo.value.Data 
        break;
      case this.optionType.SLIDELASERPENINKPOINTSCHANGED.name:
        // 当监听到激光笔的墨迹事件时拿到回调数据后直接调用
        await this.wspApplication.ActivePresentation.SlideShowWindow.View.SetLaserPenData({ Data: optionInfo.value.Data })
        break;
      case this.optionType.SLIDEINKTOOLBARVISIBLE.name:
        // this.wspApplication.ActivePresentation.SlideShowWindow.View.PointerVisible  = optionInfo.value.Data.show;
        this.wspApplication.ActivePresentation.SlideShowWindow.View.MarkerEditVisible = optionInfo.value.Data.show;
        break;
      case this.optionType.SLIDEMEDIACHANGED.name:
        await this.wspApplication.ActivePresentation.SlideShowWindow.View.SetMediaObj({ Data: optionInfo.value.Data });
        break;
      default:
        break;
    }
  }

  /**
   * 消息推送
   * @param data 推送信息
   */
  postMessage = (data: any) => {
    const { meetId, user } = this.meetingInfo;
    this.wsServer.send(JSON.stringify({id: user.id, meetId, data}));
  }

  // 播放设置右键\hover\链接是否显示
  async _setMenusVisible(flag: boolean) {
    const linkTip = this.wspApplication.Enum.PpToolType.pcPlayHoverLink // hover超链接
    const imageTip = this.wspApplication.Enum.PpToolType.pcImageHoverTip // hover 图片
    const menu = this.wspApplication.Enum.PpToolType.pcPlayingMenu // 右键菜单
    await this.wspApplication.ActivePresentation.SlideShowWindow.View.SetToolVisible(linkTip, flag)
    await this.wspApplication.ActivePresentation.SlideShowWindow.View.SetToolVisible(imageTip, flag)
    await this.wspApplication.ActivePresentation.SlideShowWindow.View.SetToolVisible(menu, flag)
  }

  /**
   * 设置slide与animate信息同步
   * @param optionInfo 同步信息
   */
  _setPageAndAnimate = async (optionInfo) => {
    const slideIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideIndex
    const clickIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.GetClickIndex()
    if (slideIndex !== optionInfo.value.Data.slideIndex + 1) {
      await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoSlide(optionInfo.value.Data.slideIndex + 1)
    }
    if (clickIndex !== optionInfo.value.Data.animateIndex + 1) {
      await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoClick(optionInfo.value.Data.animateIndex + 2)
    }
  }
}
```