```js
// 表格对象
class sheet {
  wsServer: WebSocket;
  meetingInfo: any;
  wspApplication: any;
  optionType = {
    WORKSHEET_SELECTIONCHANGE: {name: 'Worksheet_SelectionChange'}, // 选区改变
    WORKSHEET_SCROLLCHANGE: {name: 'Worksheet_ScrollChange'}, // 滚动
    WORKSHEET_FORCELANDSCAPE: {name: 'Worksheet_ForceLandscape'}, // 强制横屏时通知
  };

  constructor(wsServer: WebSocket, meetingInfo: any, wspApplication: any) {
    this.wsServer = wsServer;
    this.meetingInfo = meetingInfo;
    this.wspApplication = wspApplication;
  }

  // 初始化主讲人表格事件监听
  speakerInit() {
    Object.keys(this.optionType).forEach((key) => {
      this.wspApplication.Sub[this.optionType[key].name] = async (e:any) => {
        if (this.optionType[key].name === this.optionType.WORKSHEET_SCROLLCHANGE.name) {
          const scrollColumn = await this.wspApplication.ActiveWindow.ScrollColumn;
          const scrollRow = await this.wspApplication.ActiveWindow.ScrollRow;
          this.postMessage({type: this.optionType[key].name, value: {column: scrollColumn, row:scrollRow}});
        } else if(this.optionType[key].name === this.optionType.WORKSHEET_SELECTIONCHANGE.name) {
          // 获取选区信息
          const params = await this._getSelectionData();
          this.postMessage({type: this.optionType[key].name, value: params});
        } else {
          this.postMessage({type: this.optionType[key].name, value: e});
        }
      }
    })
  }

  async sendInitInfo() {
    const scrollColumn = await this.wspApplication.ActiveWindow.ScrollColumn;
    const scrollRow = await this.wspApplication.ActiveWindow.ScrollRow;
    const params = await this._getSelectionData()
    this.postMessage({
      type: 'init', 
      value: {
        [this.optionType.WORKSHEET_SCROLLCHANGE.name]: {column: scrollColumn, row:scrollRow},
        [this.optionType.WORKSHEET_SELECTIONCHANGE.name]: params
      }
    });
  }

  // 初始化听众响应操作
  async listenerInit(optionInfo: any) {
    switch(optionInfo.type) {
      case this.optionType.WORKSHEET_SELECTIONCHANGE.name: 
        await this.wspApplication.Sheets(optionInfo.value.index).Activate()
        if (optionInfo.value.selection.type === 'shape') {
          await this.wspApplication.ActiveSheet.Shapes.Item(1).Select()
        } else {
          await this.wspApplication.Range(optionInfo.value.selection.value).Select() // 多个单元格选中
        }
        break;
      case this.optionType.WORKSHEET_SCROLLCHANGE.name:
        this.wspApplication.ActiveWindow.ScrollColumn = optionInfo.value.column;
        this.wspApplication.ActiveWindow.ScrollRow = optionInfo.value.row;
        break;
      case this.optionType.WORKSHEET_FORCELANDSCAPE.name:
        // await WPSOpenApi.Application.Range('C47:D54').Select() // 多个单元格选中
        this.wspApplication.ActiveDocument.Range.SetRange(optionInfo.value.begin, optionInfo.value.end) // Start: number, End: number
        break;
      default:
        break;
    }
  }

  async _getSelectionData() {
    const params: {
      value: string,
      type: string
    } = {
      value: '',
      type: ''
    };
    const shapes = await this.wspApplication.Selection.Item(1).ID
    // 判断当前是选中对象还是选中单元格
    if (shapes) {
      params.value = shapes
      params.type = 'shape'
    } else {
      const selection = await this.wspApplication.Selection.Address()
      params.value = selection
      params.type = 'cell'
    }
    
    // 获取激活的sheet页
    const sheetIndex = await this.wspApplication.ActiveSheet.Index;
    // 获取激活的单元格
    const activitiItem = {row: await this.wspApplication.Selection.Row, col: await this.wspApplication.Selection.Column};
    
    return {index: sheetIndex, selection: params, activitiItem};
  }

  // 消息推送
  postMessage = (data: any) => {
    const { meetId, user } = this.meetingInfo;
    this.wsServer.send(JSON.stringify({id: user.id, meetId, data}));
  }
}
```