```js
// 白板对象
class whiteBoard {
  wsServer: WebSocket;
  meetingInfo: any;
  wspApplication: any;
  optionType = {
    DOCRESIZE: {name: 'DocResize'}, // 画布缩放量和偏移量变化
    SHAPESELECTCHANGE: {name: 'ShapeSelectChange'}, // 对象选区变化通知，选中对象的改变
    ACTIVESLIDECHANGE: {name: 'ActiveSlideChange'}, // 当前画布数据变化，包括选区变化；
  };

  constructor(wsServer: WebSocket, meetingInfo: any, wspApplication: any) {
    this.wsServer = wsServer;
    this.meetingInfo = meetingInfo;
    this.wspApplication = wspApplication;
  }

  // 初始化主讲人白板事件监听
  speakerInit() {
    Object.keys(this.optionType).forEach((key) => {
      this.wspApplication.Sub[this.optionType[key].name] = async (e: any) => {
        this.postMessage({type: this.optionType[key].name, value: e});
      }
    })
  }
  
  async sendInitInfo() {
    const viewInfo = await this.wspApplication.ActiveWhiteBoard.GetViewInfo();
    this.postMessage({type: 'init', value: {[this.optionType.DOCRESIZE.name]: {viewInfo}}});
  }

  // 初始化听众响应操作
  async listenerInit(optionInfo: any) {
    switch(optionInfo.type) {
      case this.optionType.DOCRESIZE.name: {
        await this.wspApplication.ActiveWhiteBoard.SetViewByViewInfo(optionInfo.value.viewInfo)
        break;
      }
      case this.optionType.SHAPESELECTCHANGE.name:
        this.wspApplication.ActiveWhiteBoard.SetViewByShapeIds(optionInfo.value.shapeIds)
        break;
      case this.optionType.ACTIVESLIDECHANGE.name:
        // await WPSOpenApi.Application.Range('C47:D54').Select() // 多个单元格选中
        // this.wspApplication.ActiveDocument.Range.SetRange(optionInfo.value.begin, optionInfo.value.end) // Start: number, End: number
        break;
      default:
        break;
    }
  }

  // 消息推送
  postMessage = (data: any) => {
    const { meetId, user } = this.meetingInfo;
    this.wsServer.send(JSON.stringify({id: user.id, meetId, data}));
  }
}

```