```js
// 文字对象
class writer {
  wsServer: WebSocket;
  meetingInfo: any;
  wspApplication: any;
  cacheInfo = {}; // 缓存文档操作
  optionType = {
    WINDOWSCROLLCHANGE: {name: 'WindowScrollChange'}, // 滚动通知
    WINDOWSELECTIONCHANGE: {name: 'WindowSelectionChange'} // 选区变化通知
  };

  constructor(wsServer: WebSocket, meetingInfo: any, wspApplication: any) {
    this.wsServer = wsServer;
    this.meetingInfo = meetingInfo;
    this.wspApplication = wspApplication;
  }

  // 初始化主讲人文字事件监听
  speakerInit() {
    Object.keys(this.optionType).forEach((key) => {
      this.wspApplication.Sub[this.optionType[key].name] = async (e: any) => {
        this.cacheInfo[this.optionType[key].name] = e;
        this.postMessage({type: this.optionType[key].name, value: e})
      }
    })
  }

  async sendInitInfo() {
    this.postMessage({
      type: 'init', 
      value: this.cacheInfo
    });
  }

  // 初始化听众响应操作
  async listenerInit(optionInfo: any) {
    switch(optionInfo.type) {
      case this.optionType.WINDOWSCROLLCHANGE.name: {
        const range = await this.wspApplication.ActiveDocument.ActiveWindow.RangeFromPoint(optionInfo.value.Data.scrollLeft, optionInfo.value.Data.scrollTop)
        this.wspApplication.ActiveDocument.ActiveWindow.ScrollIntoView(range)
        break;
      }
      case this.optionType.WINDOWSELECTIONCHANGE.name: {
        // 选中区域时点时设置选区为0，即不设置选区范围
        if (optionInfo.value.isPoint) {
          this.wspApplication.ActiveDocument.Range.SetRange(0, 0) // Start: number, End: number
        } else {
          this.wspApplication.ActiveDocument.Range.SetRange(optionInfo.value.begin, optionInfo.value.end) // Start: number, End: number
        }
        break;
      }
      default:
        break;
    }
  }

   // 消息推送
   postMessage = (data: any) => {
    const { meetId, user } = this.meetingInfo;
    this.wsServer.send(JSON.stringify({id: user.id, meetId, data}));
  }
}
```