import React from 'react';
import styles from './index.module.less';
import ReactMarkdown from 'react-markdown';
import gfm from 'remark-gfm';
import { PrismAsyncLight as SyntaxHighlighter } from 'react-syntax-highlighter';
// @ts-ignore-next-line
import coy from 'react-syntax-highlighter/dist/cjs/styles/prism/coy-without-shadows';
import js from 'react-syntax-highlighter/dist/cjs/languages/prism/javascript';
import jsx from 'react-syntax-highlighter/dist/cjs/languages/prism/jsx';
import go from 'react-syntax-highlighter/dist/cjs/languages/prism/go';
import java from 'react-syntax-highlighter/dist/cjs/languages/prism/java';
import json from 'react-syntax-highlighter/dist/cjs/languages/prism/json';
import { message, Tooltip } from 'antd';
import CopyToClipboard from 'react-copy-to-clipboard';
import { CopyOutlined } from '@ant-design/icons';

// 支持语法
const supportedLanguages:{[prop: string]: any} = {
  js,
  go,
  java,
  json,
  javascript: js,
  html: jsx,
};
Object.keys(supportedLanguages).forEach((key: string) => {
  SyntaxHighlighter.registerLanguage(key, supportedLanguages[key]);
});

const renderers = {
  code: (data:{ language: string, value: string }): React.ReactElement => {
    const { language, value } = data;
    const lang = supportedLanguages[language] ? language : 'text';

    // 检测到 "//@file=文件名" 注释开头的代码块才会出现试一试
    const matchFileRegExp = /\/\/@file=(.*)\r*\n*/;
    const code = value.replace(matchFileRegExp, '');

    return (
      <div className={styles.code}>
        <SyntaxHighlighter
          showLineNumbers={true}
          language={lang}
          style={coy}
          children={code}
          wrapLongLines={true}
          wrapLines={true}
        />
        <div className={styles['code-toolbox']}>
          <div className={styles.item}>
            <Tooltip title={'copyCode'}>
              <CopyToClipboard text={code} onCopy={() => message.success('copySuccess')}>
                <CopyOutlined />
              </CopyToClipboard>
            </Tooltip>
          </div>
        </div>
      </div>
    );
  },
};

export default function Content(props: {content: string, className?: string}): JSX.Element {
  return (
    <div
      className={
        props.className
        ? `${styles['doc-content']} ${props.className}`
        : styles['doc-content']
      }
    >
      <ReactMarkdown plugins={[gfm]} renderers={renderers} children={props.content} />
    </div>
  );
}
