import React from 'react';
import { message, Spin } from 'antd';
import { initActiviteClass } from '../../../utils/util';
import request from '../../../utils/request';
import styles from './index.module.less';

interface SpectatorProps {
  match: React.ComponentProps<any>;
}

let connectMessage = null; //socket重连提示 

const optionCache = [];  // 消息缓存
let optionIndex = 0;
let operationing = false;

export default class Spectator extends React.Component<SpectatorProps> {

  wsServer = null;
  socketTimer = null;
  reconnectTimer = null;
  activiteObject = null;
  wpsInstantce = null;
  state = {
    user: null,
    meetId: '',
    url: '',
    loading: false
  }

  componentDidMount() {
    this.fetchMeetInfo();
  }

  componentWillUnmount() {
    this.socketTimer && clearInterval(this.socketTimer);
    this.reconnectTimer && clearTimeout(this.reconnectTimer);
  }

  // 查询会议信息，加入会议
  fetchMeetInfo = async () => {
    const { match: { params } } = this.props;

    const data = await request(`/api/fetchMeetInfo?meetId=${params.id}`);
    this.setState({
      url: data.url,
      loading: true
    }, () => {
      this.init();
    });
  }

  init = async () => {
    await this.initWap();
    const data = await request("/api/startMeeting");
    this.setState({
      user: data
    }, () => {
      this.initWsSocket();
    })
  }

  initWap = async () => {
    const { url } = this.state;
    const wps = new window.WebOfficeSDK({
      mount: document.getElementById('listenerContent'),
      wpsUrl: url,
      commonOptions: {
        isShowHeader: false,
        isIframeViewFullscreen: true
      },
      readOnly: true,
    })
    await wps.setToken({token: '123'}) // 设置token
    await wps.advancedApiReady();
    this.wpsInstantce = wps;
  }

  initWsSocket = () => {
    if (!this.wsServer) {
      this.webSocketConnect(() => {
        this.onWebSocketCallback();
      });
    } else {
      this.onWebSocketCallback();
    }
  }

  webSocketConnect = (wsOpenCallback: () => void) => {
    const { user } = this.state;
    const { match: { params } } = this.props;
    const { TYPE, PORT } = user.wsConfig;
    const host = window.location.host;
    const ip = host.split(':')[0];
    let wsUrl = '';
    if (PORT) {
      wsUrl = `${TYPE}://${ip}:${PORT}/scene/web?type=listener&meetId=${params.id}&id=${user.id}`;
    } else {
      wsUrl = `${TYPE}://${ip}/scene/web?type=listener&meetId=${params.id}&id=${user.id}`;
    }
    this.wsServer = new WebSocket(wsUrl);
    this.wsServer.onopen = () => {
      wsOpenCallback && wsOpenCallback();

      this.wsServer.send(JSON.stringify({id: user.id, meetId: params.id, status: 'success'}));
      this.socketTimer = setInterval(() => {
        this.wsServer.send('ping');
      }, 60 * 1000)
    }
    this.wsServer.onmessage = async (e: MessageEvent) => {
      try {
        const optionInfo = JSON.parse(e.data);
        const value = optionInfo.value || [];
        if (optionInfo.type === 'change_doc_url') {
          location.reload();
        } else if (optionInfo.type === 'init') {
          Object.keys(value).forEach(key => {
            optionCache.push({type: key, value: value[key]});
          })
        } else {
          optionCache.push(optionInfo);
        }

        // 执行操作
        this.executeCacheOption();
      } catch (e) {
        console.error(e);
      }
    }
    this.wsServer.onclose = (e: CloseEvent) => {
      console.log("webscoket close", e);
      this.reconnect();
    }
    this.wsServer.onerror = (e: Event) => {
      console.log("webscoket error", e);
      this.reconnect();
    }
  }
  // 连接成功后同步演讲者信息
  onWebSocketCallback = async () => { 
    const { user } = this.state;
    const { match: { params } } = this.props;
    const activiteClass = initActiviteClass(this.wpsInstantce);
    this.activiteObject = new activiteClass(this.wsServer, {meetId: params.id, user}, this.wpsInstantce.Application);
    this.setState({
      loading: false
    });
  }

  // 执行操作
  executeCacheOption = async () => {
    if (operationing) {
      return;
    }
    operationing = true;
    while(optionIndex < optionCache.length) {
      const option = optionCache[optionIndex];
      await this.activiteObject.listenerInit(option);
      optionIndex++;
    }
    operationing = false;
  }


  // webScoket断线重连
  reconnect = () => {
    if (!connectMessage) {
      connectMessage = true;
      message.error({content: '连接异常，请稍后重试！', key: 'wsMessage'});
    }

    this.socketTimer && clearInterval(this.socketTimer);
    this.reconnectTimer && clearTimeout(this.reconnectTimer);
    this.reconnectTimer = setTimeout(() => {
      this.webSocketConnect(() => {
        this.activiteObject.wsServer = this.wsServer;
        connectMessage = null;
        console.log('ws重连成功!');
      });
    }, 10000)
  }

  render(): React.ReactNode {
    const { loading } = this.state;

    return (
      <Spin spinning={loading} tip="加载中……">
        <div id="listenerContent" className={styles.wspContent}></div>
      </Spin>
    )
  }
}