import React from 'react';
import { Select, Spin, Button } from 'antd';
import PageHeader from '../../../components/pageHeader';
import Content from '../../../components/Content';
import request from '../../../utils/request';
import Speaker from './Speaker';
import Listener from './Listener';
import styles from './index.module.less';

import config from 'config';
import speakerImg from '../../../assets/speaker.png';
import listenerImg from '../../../assets/listener.png';

const wpsFileId = config.wpsFileId;
const Option = Select.Option;

interface MeetingProps {
  history: React.ComponentProps<any>;
}

export default class HomePage extends React.Component<MeetingProps> {

  state = {
    type: 'presentation',
    markdown: null,
    wpsUrl: '',
    meetId: '',
    speakerLoading: false,
    listenerLoading: false
  }

  componentDidMount() {
    this.getFileUrl();
  }

  onHandleChangeType = (value: string) => {
    const { meetId } = this.state;
    this.updateDocUrl(meetId, value);
  }

  getFileUrl = async () => {
    const { type } = this.state;
    const data = await request(`/pnode/getUrlAndToken?fileid=${wpsFileId[type]}`);
    this.setState({
      wpsUrl: data.wpsUrl
    }, () => {
      this.createMeetId();
    });
  }

  // 查询会议Id,开启会议
  createMeetId = async () => {
    const data = await request('/api/createMeetId');
    this.setState({
      meetId: data && data.id
    }, () => {
      const { meetId, type } = this.state;
      this.updateDocUrl(meetId, type);
    });
  }

  // 更新文档信息
  updateDocUrl = async (meetId: string, type: string) => {
    const data = await request(`/pnode/getUrlAndToken?fileid=${wpsFileId[type]}`);
    await request('/api/updateMeetDocInfo',{method: 'POST', body: {meetId, url: data.wpsUrl}});
    this.fetctDoc(data.wpsUrl, type);
  }

  fetctDoc = (url: string, type: string) => {
    this.setState({
      speakerLoading: true,
      listenerLoading: true
    });
    fetch(`./doc/${type}.md`)
      .then(res => res.text())
      .then(text => {
        this.setState({ 
          markdown: text,
          wpsUrl: url,
          type
        })
      });
  }

  spealerLoadSuccess = () => {
    this.setState({
      speakerLoading: false
    });
  }

  listenerLoadSuccess = () => {
    this.setState({
      listenerLoading: false
    });
  }

  onHandleCopyLink = () => {
    const text = document.getElementById('meet-url');
    if (window.getSelection) {
        const selection = window.getSelection();
        const range = document.createRange();
        range.selectNodeContents(text);
        selection.removeAllRanges();
        selection.addRange(range);
        document.execCommand("Copy")     
      } else {
        console.error('浏览器不支持复制，请手动复制');
      }
  }

  render(): React.ReactNode {
    const { type, markdown, meetId, speakerLoading, listenerLoading, wpsUrl } = this.state;

    return (
      <Spin spinning={speakerLoading || listenerLoading} tip="加载中……">
        <PageHeader title="会议场景" />
        <div className={styles.demoContent}>
          <div className={styles.selectDiv}>
            <span>文档切换：</span>
            <Select className={styles.select} onChange={this.onHandleChangeType} value={type}>
              <Option value="pdf">pdf文档</Option>
              <Option value="presentation">演示文档</Option>
              <Option value="writer">文字文档</Option>
              <Option value="sheet">表格文档</Option>
              {/* <Option value="whiteBoard">白板文档</Option> */}
            </Select>
            <div className={styles.meetUrlDiv}>
              <span>会议链接：</span>
              <span id="meet-url">{location.origin + location.pathname +'#/listener/' + meetId}</span>
              <Button type="link" onClick={this.onHandleCopyLink}>复制</Button>
            </div>
          </div>
          <div className={styles.showArea}>
            <div className={styles.speakerArea}>
              <div className={styles.meetingTitle}>
                <img src={speakerImg} className={styles.personImg}/>
                <span>主讲人</span>
              </div>
              {meetId && <Speaker url={wpsUrl} meetId={meetId} loadSuccess={this.spealerLoadSuccess}/>}
            </div>
            <div className={styles.lineDiv}></div>
            <div className={styles.listenerArea}>
              <div className={styles.meetingTitle}>
                <img src={listenerImg} className={styles.personImg}/>
                <span>听众</span>
              </div>
              {meetId && <Listener url={wpsUrl} meetId={meetId} loadSuccess={this.listenerLoadSuccess}/>}
            </div>
          </div>
          <div className={styles.docDiv}>
            <Content content={markdown}/>
          </div>
        </div>
      </Spin>
    )
  }
}