import React from 'react';
import { Switch, Route, Redirect } from 'react-router';
import { Provider } from 'react-redux'
import * as routerRedux from 'connected-react-router';
import configure from './middleware/configureStore';
import menus from './common/menu';
import history from './history';

const { ConnectedRouter, connectRouter, routerMiddleware: createRouterMiddleware} = routerRedux;

export const routerReducer = connectRouter(history);
export const routerMiddleware = createRouterMiddleware(history);

const store = configure();

export default (): React.ReactElement => (
  <Provider store={store}>
    <ConnectedRouter history={history}>
        <Switch>
          {
            menus.map(item => {
              return <Route key={item.path} path={item.path} component={item.component} />
            })
          }
          <Route exact path="/">
            <Redirect from="/*" to="/index" />
          </Route>
        </Switch>
    </ConnectedRouter>
  </Provider>
)