// 初始化监听对象
export function initActiviteClass(wpsInstantce: any): any {
  // @ts-ignore: Unreachable code error
  switch(wpsInstantce.officeType) {
    case 'f':
      return pdf;
    case 'w':
      return writer;
    case 's':
      return sheet;
    case 'p':
      return presentation;
    case 'b':
      return whiteBoard;
  }
}


// PDF对象
class pdf {
  wsServer: WebSocket;
  meetingInfo: any;
  wspApplication: any;
  optionType = {
    ZOOM: {name: 'ZoomUpdated'},  // 缩放
    SCROLL: {name: 'Scroll'}, // 滚动
    PAGECHANGE: {name: 'CurrentPageChange'}, // 页面切换
    PAGESTARTPLAY: {name: 'StartPlay'}, // 进入播放
    PAGEENDPLAY: {name: 'EndPlay'} // 退出播放
  };
  isPlayStatus: boolean;

  constructor(wsServer: WebSocket, meetingInfo: any, wspApplication: any) {
    this.wsServer = wsServer;
    this.meetingInfo = meetingInfo;
    this.wspApplication = wspApplication;
  }

  // 初始化主讲人PDF事件监听
  speakerInit() {
    Object.keys(this.optionType).forEach((key) => {
      this.wspApplication.Sub[this.optionType[key].name] = async (e: any) => {
        let message = e;
        if (this.optionType[key].name === this.optionType.SCROLL.name) {
          const zoom = await this.wspApplication.ActivePDF.Zoom;
          message = {scroll: e, zoom};
        } else if (this.optionType[key].name === this.optionType.ZOOM.name) {
          const scroll = await this.wspApplication.ActivePDF.Scroll;
          message = {scroll, zoom: e};
        }
        this.postMessage({type: this.optionType[key].name, value: message});
      }
    })
  }

  async sendInitInfo() {
    const zoom = await this.wspApplication.ActivePDF.Zoom;
    const scroll = await this.wspApplication.ActivePDF.Scroll;
    const playMode = await this.wspApplication.ActivePDF.PlayMode;
    this.postMessage({
      type: 'init', 
      value: {
        [this.optionType.ZOOM.name]: {zoom, scroll},
        ...(playMode?
          {[this.optionType.PAGESTARTPLAY.name]: true}:
          {[this.optionType.PAGEENDPLAY.name]: true}
        )
      }
    });
  }

  // 初始化听众响应操作
  async listenerInit(optionInfo: any) {
    switch(optionInfo.type) {
      case this.optionType.SCROLL.name: {
        const zoom = await this.wspApplication.ActivePDF.Zoom;
        const x = optionInfo.value.scroll.ScrollX / optionInfo.value.zoom * zoom;
        const y = optionInfo.value.scroll.ScrollY / optionInfo.value.zoom * zoom;
        this.wspApplication.ActivePDF.ScrollTo(x, y);
        break;
      }
      case this.optionType.ZOOM.name:
        this.wspApplication.ActivePDF.Zoom = optionInfo.value.zoom;
         // 缩放后设置滚动位置
        this.wspApplication.ActivePDF.ScrollTo(optionInfo.value.scroll.x, optionInfo.value.scroll.y);
        break;
      case this.optionType.PAGECHANGE.name:
        // 播放模式下切页有效
        if (this.isPlayStatus) {
          this.wspApplication.ActivePDF.JumpToPage({PageNum: optionInfo.value + 1});
        }
        break;
      case this.optionType.PAGESTARTPLAY.name:
        this.isPlayStatus = true;
        this.wspApplication.ActivePDF.StartPlay('active',true,true)
        break;
      case this.optionType.PAGEENDPLAY.name:
        this.isPlayStatus = false;
        this.wspApplication.ActivePDF.EndPlay()
        break;
      default:
        break;
    }
  }

  // 消息推送
  postMessage = (data: any) => {
    const { meetId, user } = this.meetingInfo;
    this.wsServer.send(JSON.stringify({id: user.id, meetId, data}));
  }
}

// 文字对象
class writer {
  wsServer: WebSocket;
  meetingInfo: any;
  wspApplication: any;
  cacheInfo = {}; // 缓存文档操作
  optionType = {
    WINDOWSCROLLCHANGE: {name: 'WindowScrollChange'}, // 滚动通知
    WINDOWSELECTIONCHANGE: {name: 'WindowSelectionChange'} // 选区变化通知
  };

  constructor(wsServer: WebSocket, meetingInfo: any, wspApplication: any) {
    this.wsServer = wsServer;
    this.meetingInfo = meetingInfo;
    this.wspApplication = wspApplication;
  }

  // 初始化主讲人文字事件监听
  speakerInit() {
    Object.keys(this.optionType).forEach((key) => {
      this.wspApplication.Sub[this.optionType[key].name] = async (e: any) => {
        this.cacheInfo[this.optionType[key].name] = e;
        this.postMessage({type: this.optionType[key].name, value: e})
      }
    })
  }

  async sendInitInfo() {
    this.postMessage({
      type: 'init', 
      value: this.cacheInfo
    });
  }

  // 初始化听众响应操作
  async listenerInit(optionInfo: any) {
    switch(optionInfo.type) {
      case this.optionType.WINDOWSCROLLCHANGE.name: {
        const range = await this.wspApplication.ActiveDocument.ActiveWindow.RangeFromPoint(optionInfo.value.Data.scrollLeft, optionInfo.value.Data.scrollTop)
        this.wspApplication.ActiveDocument.ActiveWindow.ScrollIntoView(range)
        break;
      }
      case this.optionType.WINDOWSELECTIONCHANGE.name:
        this.wspApplication.ActiveDocument.Range.SetRange(optionInfo.value.begin, optionInfo.value.end) // Start: number, End: number
        break;
      default:
        break;
    }
  }

   // 消息推送
   postMessage = (data: any) => {
    const { meetId, user } = this.meetingInfo;
    this.wsServer.send(JSON.stringify({id: user.id, meetId, data}));
  }
}

// 表格对象
class sheet {
  wsServer: WebSocket;
  meetingInfo: any;
  wspApplication: any;
  optionType = {
    WORKSHEET_SELECTIONCHANGE: {name: 'Worksheet_SelectionChange'}, // 选区改变
    WORKSHEET_SCROLLCHANGE: {name: 'Worksheet_ScrollChange'}, // 滚动
    WORKSHEET_FORCELANDSCAPE: {name: 'Worksheet_ForceLandscape'}, // 强制横屏时通知
  };

  constructor(wsServer: WebSocket, meetingInfo: any, wspApplication: any) {
    this.wsServer = wsServer;
    this.meetingInfo = meetingInfo;
    this.wspApplication = wspApplication;
  }

  // 初始化主讲人表格事件监听
  speakerInit() {
    Object.keys(this.optionType).forEach((key) => {
      this.wspApplication.Sub[this.optionType[key].name] = async (e:any) => {
        let message = e;
        if (this.optionType[key].name === this.optionType.WORKSHEET_SCROLLCHANGE.name) {
          const scrollColumn = await this.wspApplication.ActiveWindow.ScrollColumn;
          const scrollRow = await this.wspApplication.ActiveWindow.ScrollRow;
          message = {column: scrollColumn, row:scrollRow};
        } else if(this.optionType[key].name === this.optionType.WORKSHEET_SELECTIONCHANGE.name) {
          // 获取选区信息
          message = await this._getSelectionData();
        }
        this.postMessage({type: this.optionType[key].name, value: message});
      }
    })
  }

  async sendInitInfo() {
    const scrollColumn = await this.wspApplication.ActiveWindow.ScrollColumn;
    const scrollRow = await this.wspApplication.ActiveWindow.ScrollRow;
    const params = await this._getSelectionData()
    this.postMessage({
      type: 'init', 
      value: {
        [this.optionType.WORKSHEET_SCROLLCHANGE.name]: {column: scrollColumn, row:scrollRow},
        [this.optionType.WORKSHEET_SELECTIONCHANGE.name]: params
      }
    });
  }

  // 初始化听众响应操作
  async listenerInit(optionInfo: any) {
    switch(optionInfo.type) {
      case this.optionType.WORKSHEET_SELECTIONCHANGE.name: 
        await this.wspApplication.Sheets(optionInfo.value.index).Activate()
        if (optionInfo.value.selection.type === 'shape') {
          await this.wspApplication.ActiveSheet.Shapes.Item(1).Select()
        } else {
          await this.wspApplication.Range(optionInfo.value.selection.value).Select() // 多个单元格选中
        }
        break;
      case this.optionType.WORKSHEET_SCROLLCHANGE.name:
        this.wspApplication.ActiveWindow.ScrollColumn = optionInfo.value.column;
        this.wspApplication.ActiveWindow.ScrollRow = optionInfo.value.row;
        break;
      case this.optionType.WORKSHEET_FORCELANDSCAPE.name:
        // await WPSOpenApi.Application.Range('C47:D54').Select() // 多个单元格选中
        this.wspApplication.ActiveDocument.Range.SetRange(optionInfo.value.begin, optionInfo.value.end) // Start: number, End: number
        break;
      default:
        break;
    }
  }

  async _getSelectionData() {
    const params: {
      value: string,
      type: string
    } = {
      value: '',
      type: ''
    };
    const shapes = await this.wspApplication.Selection.Item(1).ID
    // 判断当前是选中对象还是选中单元格
    if (shapes) {
      params.value = shapes
      params.type = 'shape'
    } else {
      const selection = await this.wspApplication.Selection.Address()
      params.value = selection
      params.type = 'cell'
    }
    
    // 获取激活的sheet页
    const sheetIndex = await this.wspApplication.ActiveSheet.Index;
    // 获取激活的单元格
    const activitiItem = {row: await this.wspApplication.Selection.Row, col: await this.wspApplication.Selection.Column};
    
    return {index: sheetIndex, selection: params, activitiItem};
  }

  // 消息推送
  postMessage = (data: any) => {
    const { meetId, user } = this.meetingInfo;
    this.wsServer.send(JSON.stringify({id: user.id, meetId, data}));
  }
}

// 演示对象
class presentation {
  wsServer: WebSocket;
  wspApplication: any;
  meetingInfo: any;
  cacheInfo = {}; // 文档操作缓存
  optionType = {
    SLIDESHOWONNEXT: {name: 'SlideShowOnNext'}, // 播放模式下一步触发
    SLIDESHOWONPREVIOUS: {name: 'SlideShowOnPrevious'}, // 播放模式上一步触发
    SLIDEMEDIACHANGED: {name: 'SlideMediaChanged'}, // 视频播放状态改变时触发
    SLIDEPLAYERCHANGE: {name: 'SlidePlayerChange'}, // 播放状态改变时触发
    SLIDELASERPENINKPOINTSCHANGED: {name: 'SlideLaserPenInkPointsChanged'}, // 发送激光笔的墨迹
    SLIDESHOWBEGIN: {name: 'SlideShowBegin'}, // 进入播放 
    SLIDESHOWEND: {name: 'SlideShowEnd'}, // 退出播放 
    SLIDEINKVISIBLE: {name: 'SlideInkVisible'}, // 是否显示标注
    SLIDEINKTOOLBARVISIBLE: {name: 'SlideInkToolbarVisible'}, // 是否使用激光笔和标注
  };

  constructor(wsServer: WebSocket, meetingInfo: any, wspApplication: any) {
    this.wsServer = wsServer;
    this.meetingInfo = meetingInfo;
    this.wspApplication = wspApplication;
  }

  // 初始化主讲人演示事件监听
  speakerInit() {
    Object.keys(this.optionType).forEach(async (key) => {
      this.wspApplication.Sub[this.optionType[key].name] = async (e: any) => {
        let message = e;
        if (this.optionType[key].name === this.optionType.SLIDESHOWBEGIN.name) {
          // 初始化播放页码与动画
          const slideIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideIndex
          message = {slideIndex, clickIndex: 0};
        }
        this.postMessage({type: this.optionType[key].name, value: message})
        this.cacheInfo[this.optionType[key].name] = message;
        // 播放模式隐藏右键菜单\hover\链接
        if (this.optionType[key].name === this.optionType.SLIDESHOWBEGIN.name) {
          delete this.cacheInfo[this.optionType.SLIDESHOWEND.name];
          await this._setMenusVisible(false);
        } else if (this.optionType[key].name === this.optionType.SLIDESHOWEND.name) {
          delete this.cacheInfo[this.optionType.SLIDESHOWBEGIN.name];
          await this._setMenusVisible(true);
        }
      }
    })
  }
  /**
   * 主讲人初始化消息推送至听众
   * @param isPlay 是否开启播放模式
   */
  async sendInitInfo(isPlay?: boolean) {
    // 演示文档初始化后自动进入播放模式
    const params = {};
    if (isPlay) {
      await this.wspApplication.ActivePresentation.SlideShowSettings.Run();
      const slideIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideIndex;
      const clickIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.GetClickIndex();
      this.cacheInfo[this.optionType.SLIDESHOWBEGIN.name] = {slideIndex, clickIndex};
      params[this.optionType.SLIDESHOWBEGIN.name] = {slideIndex, clickIndex};
    } else if (this.cacheInfo[this.optionType.SLIDESHOWBEGIN.name]) {
      const slideIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideIndex;
      const clickIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.GetClickIndex();
      this.cacheInfo[this.optionType.SLIDESHOWBEGIN.name] = {slideIndex, clickIndex};
      params[this.optionType.SLIDESHOWBEGIN.name] = {slideIndex, clickIndex};
    }
    
    // 除页面页码、动画设置外，其它初始化根据缓存设置
    const filterKey = [
      this.optionType.SLIDESHOWONNEXT.name,
      this.optionType.SLIDESHOWONPREVIOUS.name,
      this.optionType.SLIDEPLAYERCHANGE.name
    ]; 
    Object.keys(this.cacheInfo).forEach(key => {
      if (filterKey.indexOf(key) < 0 ) {
        params[key] = this.cacheInfo[key];
      }
    })
    this.postMessage({type: 'init', value: params});
  }

  // 初始化听众响应操作
  async listenerInit(optionInfo: any) {
    switch(optionInfo.type) {
      case this.optionType.SLIDESHOWONNEXT.name: {
        //await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoNextClick()
        break;
      }
      case this.optionType.SLIDESHOWONPREVIOUS.name: {
        //await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoPreClick()
        break;
      }
      case this.optionType.SLIDEPLAYERCHANGE.name: {
        const playMode = await this.wspApplication.ActivePresentation.SlideShowWindow.View.State;
        const slideId = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideID;
        const slideIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideIndex;
        if (slideId === optionInfo.value.Data.slideId
              && slideIndex !== optionInfo.value.Data.slideIndex + 1 || playMode !== 'play') {
          return;
        }
        
        if (optionInfo.value.Data.action === 'forward') {
           await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoNextClick()
          //  this._setPageAndAnimate(optionInfo);
        }
        if (optionInfo.value.Data.action === 'backward') {
          await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoPreClick()
          // this._setPageAndAnimate(optionInfo);
        }
        if (optionInfo.value.Data.action === 'switchTo'
            || optionInfo.value.Data.action === 'effectFinish'
            && optionInfo.value.Data.isLastEffect) {
          this._setPageAndAnimate(optionInfo);
        }
        break;
      }
      case this.optionType.SLIDESHOWBEGIN.name:
        await this.wspApplication.ActivePresentation.SlideShowSettings.Run()
        await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoSlide(optionInfo.value.slideIndex)
        await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoClick(optionInfo.value.clickIndex)
        await this._setMenusVisible(false);
        break;
      case this.optionType.SLIDESHOWEND.name:
        await this.wspApplication.ActivePresentation.SlideShowWindow.View.Exit()
        await this._setMenusVisible(true);
        break;
      case this.optionType.SLIDEINKVISIBLE.name:
        this.wspApplication.ActivePresentation.SlideShowWindow.View.PointerVisible = optionInfo.value.Data.showmark
        break;
      case this.optionType.SLIDELASERPENINKPOINTSCHANGED.name:
        // 当监听到激光笔的墨迹事件时拿到回调数据后直接调用
        await this.wspApplication.ActivePresentation.SlideShowWindow.View.SetLaserPenData({ Data: optionInfo.value.Data })
        break;
      case this.optionType.SLIDEINKTOOLBARVISIBLE.name:
        // this.wspApplication.ActivePresentation.SlideShowWindow.View.PointerVisible  = optionInfo.value.Data.show;
        this.wspApplication.ActivePresentation.SlideShowWindow.View.MarkerEditVisible = optionInfo.value.Data.show;
        break;
      case this.optionType.SLIDEMEDIACHANGED.name:
        await this.wspApplication.ActivePresentation.SlideShowWindow.View.SetMediaObj({ Data: optionInfo.value.Data });
        break;
      default:
        break;
    }
  }

  /**
   * 消息推送
   * @param data 推送信息
   */
  postMessage = (data: any) => {
    const { meetId, user } = this.meetingInfo;
    this.wsServer.send(JSON.stringify({id: user.id, meetId, data}));
  }

  // 播放设置右键\hover\链接是否显示
  async _setMenusVisible(flag: boolean) {
    const linkTip = this.wspApplication.Enum.PpToolType.pcPlayHoverLink // hover超链接
    const imageTip = this.wspApplication.Enum.PpToolType.pcImageHoverTip // hover 图片
    const menu = this.wspApplication.Enum.PpToolType.pcPlayingMenu // 右键菜单
    await this.wspApplication.ActivePresentation.SlideShowWindow.View.SetToolVisible(linkTip, flag)
    await this.wspApplication.ActivePresentation.SlideShowWindow.View.SetToolVisible(imageTip, flag)
    await this.wspApplication.ActivePresentation.SlideShowWindow.View.SetToolVisible(menu, flag)
  }

  /**
   * 设置slide与animate信息同步
   * @param optionInfo 同步信息
   */
  _setPageAndAnimate = async (optionInfo) => {
    const slideIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.Slide.SlideIndex
    const clickIndex = await this.wspApplication.ActivePresentation.SlideShowWindow.View.GetClickIndex()
    if (slideIndex !== optionInfo.value.Data.slideIndex + 1) {
      await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoSlide(optionInfo.value.Data.slideIndex + 1)
    }
    if (clickIndex !== optionInfo.value.Data.animateIndex + 1) {
      await this.wspApplication.ActivePresentation.SlideShowWindow.View.GotoClick(optionInfo.value.Data.animateIndex + 2)
    }
  }
}


// 白板对象
class whiteBoard {
  wsServer: WebSocket;
  meetingInfo: any;
  wspApplication: any;
  optionType = {
    DOCRESIZE: {name: 'DocResize'}, // 画布缩放量和偏移量变化
    SHAPESELECTCHANGE: {name: 'ShapeSelectChange'}, // 对象选区变化通知，选中对象的改变
    ACTIVESLIDECHANGE: {name: 'ActiveSlideChange'}, // 当前画布数据变化，包括选区变化；
  };

  constructor(wsServer: WebSocket, meetingInfo: any, wspApplication: any) {
    this.wsServer = wsServer;
    this.meetingInfo = meetingInfo;
    this.wspApplication = wspApplication;
  }

  // 初始化主讲人白板事件监听
  speakerInit() {
    Object.keys(this.optionType).forEach((key) => {
      this.wspApplication.Sub[this.optionType[key].name] = async (e: any) => {
        this.postMessage({type: this.optionType[key].name, value: e});
      }
    })
  }
  
  async sendInitInfo() {
    const viewInfo = await this.wspApplication.ActiveWhiteBoard.GetViewInfo();
    this.postMessage({type: 'init', value: {[this.optionType.DOCRESIZE.name]: {viewInfo}}});
  }

  // 初始化听众响应操作
  async listenerInit(optionInfo: any) {
    switch(optionInfo.type) {
      case this.optionType.DOCRESIZE.name: {
        await this.wspApplication.ActiveWhiteBoard.SetViewByViewInfo(optionInfo.value.viewInfo)
        break;
      }
      case this.optionType.SHAPESELECTCHANGE.name:
        this.wspApplication.ActiveWhiteBoard.SetViewByShapeIds(optionInfo.value.shapeIds)
        break;
      case this.optionType.ACTIVESLIDECHANGE.name:
        // await WPSOpenApi.Application.Range('C47:D54').Select() // 多个单元格选中
        // this.wspApplication.ActiveDocument.Range.SetRange(optionInfo.value.begin, optionInfo.value.end) // Start: number, End: number
        break;
      default:
        break;
    }
  }

  // 消息推送
  postMessage = (data: any) => {
    const { meetId, user } = this.meetingInfo;
    this.wsServer.send(JSON.stringify({id: user.id, meetId, data}));
  }
}
