import express, { Request, Response, NextFunction } from 'express';
import path from 'path';
import cookieParser from "cookie-parser";
import compression from 'compression';
import logger from "morgan";
import { polyfill } from 'es6-promise';
import routes from './src/routes';

polyfill();

const app = express();

// view engine setup
app.set('views', path.join(__dirname, './views'));
app.set('view engine', 'ejs');

app.use(logger('dev'));
app.use(express.json());
app.use(express.urlencoded({ extended: false }));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));
app.use(compression())

// 允许跨域
app.use(function(req, res, next) {
  res.header("Access-Control-Allow-Origin", '*');
  res.header("Access-Control-Allow-Headers", "Accept,Content-Type,X-CSRFToken,X-Requested-With");
  res.header("Access-Control-Allow-Methods","PUT,POST,GET,DELETE,OPTIONS");
  res.header("Access-Control-Allow-Credentials","true");
  res.header("Access-Control-Expose-Headers",'Accept,Content-Type,X-CSRFToken,X-Requested-With')
  next();
});

app.use('/scene', routes);

// catch 404 and forward to error handler
app.use(function(req: Request, res: Response, next: NextFunction) {
  const err: any = new Error('Page Not Found');
  err.status = 404;
  next(err);
});

// error handler
app.use(function(err: any, req: Request, res: Response) {
  // set locals, only providing error in development
  res.locals.message = err.message;
  res.locals.error = req.app.get('env') === 'development' ? err : {};
  console.log(err)
  res.status(err.status || 500);
  res.send(err.message || err.toString());
});


export default app;
