import { Request, Response } from 'express';
import { 
  mockUsersList,
  mockUsersInfo,
  getUsersInfo,
  fetchMessages,
  saveMessageInfo,
  updateMessageStatus,
  deleteMessageInfo,
  updateUser
} from '../../services/mockService';
import { WebSocketServer } from '../../services/webSocket';

async function fetchUsers(req: Request, res: Response): Promise<void> {
  const users = mockUsersList(req);
  res.json(users)
}

async function fetchUserInfoByUuid(req: Request, res: Response): Promise<void> {
  const userInfo = mockUsersInfo(req);
  res.json(userInfo)
}

async function fetchUserInfo(req: Request, res: Response): Promise<void> {
  const wsObj = new WebSocketServer();
  const userInfo = getUsersInfo();
  res.json({userInfo, wsConfig: wsObj.getWsConfig()})
}

async function fetchMessageList(req: Request, res: Response): Promise<void> {
  const userInfo = fetchMessages(req);

  res.json(userInfo)
}

async function saveMessage(req: Request, res: Response): Promise<void> {
  saveMessageInfo(req);

  res.json({success: true})
}

async function deleteMessage(req: Request, res: Response): Promise<void> {
  deleteMessageInfo(req);

  res.json({success: true})
}

async function changeMessageStatus(req: Request, res: Response): Promise<void> {
  updateMessageStatus(req);

  res.json({success: true})
}

async function updateUserInfo(req: Request, res: Response): Promise<void> {
  updateUser(req)
  res.json({success: true})
}


export {
  fetchUsers,
  fetchUserInfo,
  fetchUserInfoByUuid,
  saveMessage,
  deleteMessage,
  fetchMessageList,
  changeMessageStatus,
  updateUserInfo
}