import { Request, Response } from 'express';
import uuidv4 from 'uuid/v4';
import { WebSocketServer } from '../../services/webSocket';
import { createMeet, updateMeetInfo, getMeetInfo } from '../../services/meetService';

async function startMeeting(req: Request, res: Response): Promise<void> {
  const id = uuidv4();
  const wsObj = new WebSocketServer();
  res.json({
    wsConfig: wsObj.getWsConfig(),
    id
  })
}

async function createMeetId(req: Request, res: Response): Promise<void> {
  const id = uuidv4();
  createMeet(id);
  res.json({id})
}

async function updateMeetDocInfo(req: Request, res: Response): Promise<void> {
  const meetId = req.body.meetId;
  const url = req.body.url;
  updateMeetInfo(meetId, url)

  res.json({success: true})
}

async function fetchMeetInfo(req: Request, res: Response): Promise<void> {
  const meetId = req.query.meetId as string;
  const url = getMeetInfo(meetId)

  res.json({url})
}

export {
  createMeetId,
  startMeeting,
  updateMeetDocInfo,
  fetchMeetInfo
}