import { Router, Request, Response } from 'express';
import path from 'path';
import { asyncMiddleware } from '../util';
import { 
  startMeeting,
  fetchUsers,
  fetchUserInfo,
  fetchUserInfoByUuid,
  fetchMessageList,
  saveMessage,
  changeMessageStatus,
  createMeetId,
  fetchMeetInfo,
  updateMeetDocInfo,
  deleteMessage,
  updateUserInfo
} from '../controller';

const router = Router();

// /* GET home page. */
router.get('/', asyncMiddleware(function(req: Request, res: Response) {
  //前端获取dir文件夹下的文件名,仅供参考,开发者可以按照本身需求重新定义
  res.sendFile(path.join(__dirname, '../../public/scene', 'index.html'));
}));

// 生成会议ID
router.get("/api/createMeetId", createMeetId)

// 获取会议信息
router.get("/api/fetchMeetInfo", fetchMeetInfo)

// 更新会议文档链接
router.post("/api/updateMeetDocInfo", updateMeetDocInfo)

// 开启websocket连接,开始会议
router.get("/api/startMeeting", startMeeting)

// 查询用户数据
router.get("/api/fetchUsers", fetchUsers)

// 根据消息uuid查询用户详情
router.get("/api/fetchUserInfoByUuid", fetchUserInfoByUuid)

// 查询用户详情
router.get("/api/fetchUserInfo", fetchUserInfo)

// 更新用户
router.post("/api/updateUserInfo", updateUserInfo)

// 查询消息列表
router.get("/api/fetchMessageList", fetchMessageList)

// 存储消息推送信息
router.post("/api/saveMessage", saveMessage)

// 删除消息推送信息
router.get("/api/deleteMessage", deleteMessage)

// 取消消息推送
router.get("/api/changeMessageStatus", changeMessageStatus)


export default router;