import { connection } from 'websocket';

interface ISpeaker {
  id: string,
  conn: connection | null
}

interface IConnInfo {
  speaker: ISpeaker, // speaker 信息
  listeners: Record<string, any>[], // listener 信息
  wpsUrl: string // 会议文档链接
}

let webUsers: Record<string, IConnInfo> = {}; // websocket用户信息
let lastMessage = {}; // 存储待同步实时会议的用户

export const Meeting = {
  initialize(params: Record<string, any>, conn: connection): void {
    const { meetId, type, id } = params;
    let meetInfo = webUsers[meetId];
    if (meetInfo) {
      if (type === 'speaker') {
        meetInfo.speaker = {id, conn};
      } else {
        meetInfo.listeners.push({id, conn})
      }
    }
  },
  onMessage(messageObject: Record<string, any>, conn: connection): void {
    let meetId = messageObject.meetId || '';
    let meetInfo = webUsers[meetId];
    if (meetInfo) {
      const speaker = meetInfo.speaker;
      const listeners = meetInfo.listeners;
      
      // 主持人消息推送
      if (messageObject.id && messageObject.id === speaker.id) {
        // 初始化演讲者设置信息
        const { type } = messageObject.data || {}
        if (type === 'init') {
          (lastMessage[meetId] || []).forEach(item => {
            item.conn.sendUTF(JSON.stringify(messageObject.data))
          })
          lastMessage[meetId] = [];
        } else if(type){
          listeners.forEach(listener => {
            listener.conn.sendUTF(JSON.stringify(messageObject.data))
          })
        }
      }

      // 听众初始化后推送消息给主讲人获取主讲人最新动态
      if (messageObject.id && messageObject.id !== speaker.id) {
        if (messageObject.status === 'success') {
          if (speaker.conn) {
            speaker.conn.sendUTF(JSON.stringify({meetId, id: messageObject.id}));
          }

          lastMessage[meetId] = lastMessage[meetId] || [];
          lastMessage[meetId].push({id: messageObject.id, conn});
        }
      }
    }
  },
  closeUserWs(params: Record<string, any>): void {
    const { meetId, userType, id } = params;
    if (userType === 'speaker') {
      delete webUsers[meetId];
      delete lastMessage[meetId];
    } else {
      const meetInfo = webUsers[meetId];
      if (meetInfo) {
        const newListeners= meetInfo.listeners.filter(listener => {
          if (listener.id === id) {
            return false;
          }
  
          return true;
        });
  
        webUsers[meetId].listeners = newListeners;
      }
    }
  }
}


export function createMeet(meetId: string): void {
  const flag = Object.keys(webUsers).some(key => {
    if (key === meetId) {
      return true;
    }

    return false;
  })

  if (!flag) {
    webUsers[meetId] = {speaker: {id: '', conn: null}, listeners: [], wpsUrl: ''};
  }
}

export function updateMeetInfo(meetId: string, url: string): void {
  Object.keys(webUsers).forEach(key => {
    if (key === meetId) {
      webUsers[meetId].wpsUrl = url;
    }
  })
}

export function getMeetInfo(meetId: string): string {
  let url = '';
  Object.keys(webUsers).forEach(key => {
    if (key === meetId) {
      url = webUsers[meetId].wpsUrl;
    }
  })

  return url;
}