import { server as WsServer } from 'websocket';
import config from '../../config';
import { Meeting } from './meetService';
import { At } from './mockService';

export class WebSocketServer {

  private static httpServer;
  private wsConfig = config.WS;

  constructor() {
    this.createWebSocketServer();
  }

  createWebSocketServer(): void {
    if (!WebSocketServer.httpServer) {
      WebSocketServer.httpServer = global.httpServer;

      const wsServer = new WsServer({
        httpServer: WebSocketServer.httpServer,
        autoAcceptConnections: false
      });

      wsServer.on('request', (request) => {
        const connection = request.accept('', request.origin);
        const queryObj: Record<string, unknown> = (request.resourceURL.query || {}) as any;
        const pathname = request.resourceURL.pathname as string;
        const pathArray = pathname.split('/');
        const type = pathArray[pathArray.length - 1]
        // 根据对应的业务实例化业务对象
        let instanceObj: any = null;
        if (type === 'web') {
          instanceObj = Meeting;
        }
        if (type === 'at') {
          instanceObj = At;
        }
      
        if(!instanceObj) return;
        instanceObj.initialize(queryObj, connection)

        connection.on('message', (message) => {
          if (message.type === 'utf8') {
            let messageObject: any = {};
            try {
              messageObject = JSON.parse(message.utf8Data || "{}");
            } catch(e) {
              messageObject = {data: message.utf8Data}
            }
            
            instanceObj.onMessage(messageObject, connection);
          }
        });
        connection.on('close', () => {
          instanceObj.closeUserWs(queryObj);
          console.log((new Date()) + ' Peer ' + connection.remoteAddress + ' disconnected.');
        });

        connection.on('error', (error) => {
          instanceObj.closeUserWs(queryObj);
          console.log("Connection Error: " + error.toString());
        });
      });
    } else {
      console.log('ws已启动');
    }
  }

  getWsConfig(): Record<string, string> {
    return this.wsConfig;
  }
}